#include "DEM.h"

#if 0
#define DEBUG_generate_vertices_math_order
#define DEBUG_generate_triangle
#endif

#define DEBUG_generate_vertices
#define DEBUG_generate_triangle

void generate_vertices(int Datafile, DEM_record **Records, int LL_x, int LL_y,
                      int UR_x, int UR_y)
{
   int          M;
   int          N;
   int          i;
   int          j;
   int          Index;
   int         *Elevations;

   int          Total_Num_Vertices;
   double     **Column;
   double      *double_Pointer;
   char        *Buffer;
   int          Buffer_Size;

   M = UR_y - LL_y + 1;
   N = UR_x - LL_x + 1;

   if ((Column = (double **) malloc(M * sizeof(double *))) == NULL)
   {
      perror("generate_vertices()");
      exit(1);
   }

   Buffer_Size = M * 3 * sizeof(double);

   if ((Buffer = (char *) malloc(Buffer_Size)) == NULL)
   {
      perror("generate_vertices()");
      exit(1);
   }

   double_Pointer = (double *) Buffer;

   for (i = 0, Index = 0; i < M; i++, Index += 3)
      Column[i] = &(double_Pointer[Index]);

   Total_Num_Vertices  = N * M;

   write(Datafile, &Total_Num_Vertices, sizeof(int));

#if  defined(DEBUG_generate_vertices) || defined(DEBUG_generate_vertices_math_order)
#if 0
   fprintf(stderr, "Total_Num_Vertices = %d\n", Total_Num_Vertices);
#endif
   printf("%d\n", Total_Num_Vertices);
#endif
   for (j = LL_x; j <= UR_x; j++)
   {
      Elevations = (Records[j]) -> Elevations;
      for (i = LL_y, Index = 0; i <= UR_y; i++, Index++)
      {
         Column[Index][0] = (double) (j);
         Column[Index][1] = (double) Elevations[i];
         Column[Index][2] = (double) (i);

#ifdef DEBUG_generate_vertices_math_order
         printf("%lf %lf %lf\n", Column[Index][0], Column[Index][2],
                Column[Index][1]);
#endif
#ifdef DEBUG_generate_vertices
         printf("%lf %lf %lf\n", Column[Index][0], Column[Index][1],
                Column[Index][2]);
#endif
#ifdef PRINT_Elevations
         printf("%lf\n", Column[Index][2]);
#endif
      }
      write(Datafile, Buffer, Buffer_Size);
   }

   free(Buffer);
   free(Column);
}

/*
 *                    (M - 1, N - 1)
 *      ^             ur_x, ur_y
 *      |
 *      |
 *      |
 *      |
 *      |
 *(0, 0)|------------->
 * ll_x, ll_y
 */

void generate_triangle(int Datafile, int M, int N)
{
   int   M_Limit;
   int   N_Limit;

   int   Num_Polygons_per_Column;
   int   Total_Num_Polygons;

   int   Buffer_Size;
   char *Buffer;
   int  *int_Pointer;

   int   i;
   int   j;
   int   Index;

   int   Base;
   int   Factor;

   int **Polygons;

   M_Limit = M - 1;
   N_Limit = N - 1;

   Num_Polygons_per_Column = 2 * M_Limit;

   Total_Num_Polygons = N_Limit * Num_Polygons_per_Column;

   if ((Polygons = (int **) malloc(Num_Polygons_per_Column * sizeof(int *)))
       == NULL)
   {
      perror("generate_triangle()");
      exit(1);
   }

   Buffer_Size = Num_Polygons_per_Column * 3 * sizeof(int);

   if ((Buffer = (char *) malloc(Buffer_Size)) == NULL)
   {
      perror("generate_triangle()");
      exit(1);
   }

   int_Pointer = (int *) Buffer;

   for (i = 0, Index = 0; i < Num_Polygons_per_Column; i++, Index += 3)
      Polygons[i] = &(int_Pointer[Index]);

#ifdef DEBUG_generate_triangle
#if 0
   fprintf(stderr, "Total_Num_Polygons = %d\n", Total_Num_Polygons);
#endif
   printf("%d\n", Total_Num_Polygons);
#endif

   write(Datafile, &Total_Num_Polygons, sizeof(int));

   for (i = 0, Base = 0; i < N_Limit; i++, Base += M)
   {
      Index = 0;
      for (j = 0; j < M_Limit; j++)
      {
         Factor = Base + j;
#ifdef DEBUG_generate_triangle
         printf("%d %d %d\n", Factor, Factor + M + 1, Factor + M);
#endif
         Polygons[Index][0] = Factor;
         Polygons[Index][1] = Factor + M + 1;
         Polygons[Index][2] = Factor + M;
         Index++;

#ifdef DEBUG_generate_triangle
         printf("%d %d %d\n", Factor, Factor + 1,
                Factor + M + 1);
#endif
         Polygons[Index][0] = Factor;
         Polygons[Index][1] = Factor + 1;
         Polygons[Index][2] = Factor + M + 1;
         Index++;
      }
      write(Datafile, Buffer, Buffer_Size);
   }

   free(Buffer);
   free(Polygons);
}
